@@ -11,7 +11,8 @@ cd ${DAPTESTFILES}
 F=`ls -1 *.dap | sed -e 's/[.]dap//g' | tr '\r\n' '  '`
 cd $WD
 
-mkdir ./results_test_data
+setresultdir results_test_data
+
 if test "x${RESET}" = x1 ; then rm -fr ${BASELINE}/*.d4d ; fi
 for f in $F ; do
     echo "testing: ${f}"
@@ -24,7 +25,7 @@ for f in $F ; do
 	    failure "diff -wBb ${BASELINE}/${f}.d4d ./results_test_data/${f}.d4d"
 	fi
     elif test "x${RESET}" = x1 ; then
-	echo "${f}:" 
+	echo "${f}:"
 	cp ./results_test_data/${f}.d4d ${BASELINE}/${f}.d4d
     fi
 done
@@ -81,14 +82,12 @@ if test "x${CDLDIFF}" = x1 ; then
     trim ${CDLTESTFILES}/${STEM}.cdl ./b1
     trim ./results_test_data/${f}.d4d ./r1
     baseclean b1 b2
-    resultclean r1 r2  
+    resultclean r1 r2
     if ! diff -wBb ./b2 ./r2 ; then
-	failure "${f}" 
+	failure "${f}"
     fi
   done
 fi
 rm -rf ./results_test_data
 
 finish
-
-
