@@ -1,5 +1,5 @@
 /*********************************************************************
- *   Copyright 2009, UCAR/Unidata
+ *   Copyright 2018, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
  *********************************************************************/
 
@@ -51,8 +51,7 @@ typedef struct NCtypealignset {
     NCalignment ncvlenalign;	/* nc_vlen_t*/
 } NCtypealignset;
 
-size_t NC_class_alignment(int ncclass);
-void NC_compute_alignments(void);
-EXTERNL int NC_alignments_computed;
+EXTERNL size_t NC_class_alignment(int ncclass);
+EXTERNL void NC_compute_alignments(void);
 
 #endif /*NCOFFSETS_H*/
