@@ -25,7 +25,7 @@
 #endif
 
 /* Define the set of protocols known to be constrainable */
-static char* constrainableprotocols[] = {"http", "https",NULL};
+static const char* constrainableprotocols[] = {"http", "https",NULL};
 
 static int ncd2initialized = 0;
 
@@ -63,7 +63,7 @@ static NCerror applyclientparams(NCDAPCOMMON*);
 static int
 NCD2_create(const char *path, int cmode,
            size_t initialsz, int basepe, size_t *chunksizehintp,
-           void* mpidata, NC_Dispatch*,NC* ncp);
+           void* mpidata, const struct NC_Dispatch*,int);
 
 static int NCD2_redef(int ncid);
 static int NCD2__enddef(int ncid, size_t h_minfree, size_t v_align, size_t v_minfree, size_t r_align);
@@ -88,7 +88,7 @@ static int NCD2_get_vars(int ncid, int varid,
 	    const size_t *start, const size_t *edges, const ptrdiff_t* stride,
             void *value, nc_type memtype);
 
-static NC_Dispatch NCD2_dispatch_base = {
+static const NC_Dispatch NCD2_dispatch_base = {
 
 NC_FORMATX_DAP2,
 
@@ -101,8 +101,6 @@ NCD2_sync,
 NCD2_abort,
 NCD2_close,
 NCD2_set_fill,
-NCD2_inq_base_pe,
-NCD2_set_base_pe,
 NCD2_inq_format,
 NCD2_inq_format_extended, /*inq_format_extended*/
 
@@ -138,7 +136,6 @@ NCD2_inq_var_all,
 NCD2_var_par_access,
 NCD2_def_var_fill,
 
-#ifdef USE_NETCDF4
 NCD2_show_metadata,
 NCD2_inq_unlimdims,
 NCD2_inq_ncid,
@@ -177,11 +174,9 @@ NCD2_def_var_filter,
 NCD2_set_var_chunk_cache,
 NCD2_get_var_chunk_cache,
 
-#endif /*USE_NETCDF4*/
-
 };
 
-NC_Dispatch* NCD2_dispatch_table = NULL; /* moved here from ddispatch.c */
+const NC_Dispatch* NCD2_dispatch_table = NULL; /* moved here from ddispatch.c */
 
 int
 NCD2_initialize(void)
@@ -200,7 +195,6 @@ NCD2_initialize(void)
 int
 NCD2_finalize(void)
 {
-    curl_global_cleanup();
     return NC_NOERR;
 }
 
@@ -231,7 +225,7 @@ NCD2_abort(int ncid)
 static int
 NCD2_create(const char *path, int cmode,
            size_t initialsz, int basepe, size_t *chunksizehintp,
-           void* mpidata, NC_Dispatch* dispatch, NC* ncp)
+           void* mpidata, const NC_Dispatch* dispatch, int ncid)
 {
    return NC_EPERM;
 }
@@ -251,7 +245,7 @@ NCD2_get_vara(int ncid, int varid,
             void *value,
 	    nc_type memtype)
 {
-    int stat = nc3d_getvarx(ncid, varid, start, edges, nc_ptrdiffvector1, value,memtype);
+    int stat = nc3d_getvarx(ncid, varid, start, edges, NC_stride_one, value,memtype);
     return stat;
 }
 
@@ -275,14 +269,19 @@ NCD2_get_vars(int ncid, int varid,
 /* See ncd2dispatch.c for other version */
 int
 NCD2_open(const char* path, int mode, int basepe, size_t *chunksizehintp,
-          void* mpidata, NC_Dispatch* dispatch, NC* drno)
+          void* mpidata, const NC_Dispatch* dispatch, int ncid)
 {
     NCerror ncstat = NC_NOERR;
     OCerror ocstat = OC_NOERR;
     NCDAPCOMMON* dapcomm = NULL;
+    NC* drno;
     const char* value;
     int nc3id = -1;
 
+    /* Find pointer to NC struct for this file. */
+    ncstat = NC_check_id(ncid,&drno);
+    if(ncstat != NC_NOERR) {goto done;}
+
     if(path == NULL)
 	{ncstat = NC_EDAPURL; goto done;}
     if(dispatch == NULL)
@@ -323,7 +322,7 @@ NCD2_open(const char* path, int mode, int basepe, size_t *chunksizehintp,
     dapcomm->oc.rawurltext = strdup(path);
 #endif
 
-    if(ncuriparse(dapcomm->oc.rawurltext,&dapcomm->oc.url) != NCU_OK)
+    if(ncuriparse(dapcomm->oc.rawurltext,&dapcomm->oc.url))
 	{ncstat = NC_EURL; goto done;}
 
     if(!constrainable(dapcomm->oc.url))
@@ -1211,7 +1210,7 @@ fprintf(stderr,"basedim: %s=%ld\n",dim->ncfullname,(long)dim->dim.declsize);
 int
 constrainable(NCURI* durl)
 {
-   char** protocol = constrainableprotocols;
+   const char** protocol = constrainableprotocols;
    for(;*protocol;protocol++) {
 	if(strcmp(durl->protocol,*protocol)==0)
 	    return 1;
@@ -2237,7 +2236,7 @@ applyclientparamcontrols(NCDAPCOMMON* dapcomm)
     if(dapparamcheck(dapcomm,"show","fetch"))
 	SETFLAG(dapcomm->controls,NCF_SHOWFETCH);
 
-    /* enable/disable _FillValue/Variable Mis-match */
+    /* enable/disable _FillValue/Variable Mismatch */
     if(dapparamcheck(dapcomm,"fillmismatch",NULL))
 	SETFLAG(dapcomm->controls,NCF_FILLMISMATCH);
     else if(dapparamcheck(dapcomm,"nofillmismatch",NULL))
@@ -2256,12 +2255,6 @@ NCD2_set_fill(int ncid, int fillmode, int* old_modep)
     return THROW(NC_EPERM);
 }
 
-int
-NCD2_set_base_pe(int ncid, int pe)
-{
-    return THROW(NC_EPERM);
-}
-
 int
 NCD2_def_dim(int ncid, const char* name, size_t len, int* idp)
 {
@@ -2286,16 +2279,6 @@ NCD2_def_var(int ncid, const char *name,
 Following functions basically return the netcdf-3 value WRT to the nc3id.
 */
 
-int
-NCD2_inq_base_pe(int ncid, int* pe)
-{
-    NC* drno;
-    int ret;
-    if((ret = NC_check_id(ncid, (NC**)&drno)) != NC_NOERR) return THROW(ret);
-    ret = nc_inq_base_pe(getnc3id(drno), pe);
-    return THROW(ret);
-}
-
 int
 NCD2_inq_format(int ncid, int* formatp)
 {
@@ -2485,8 +2468,6 @@ NCD2_def_var_fill(int ncid, int p2, int p3, const void* p4)
     return THROW(ret);
 }
 
-#ifdef USE_NETCDF4
-
 int
 NCD2_inq_ncid(int ncid, const char* name, int* grp_ncid)
 {
@@ -2853,4 +2834,3 @@ NCD2_get_var_chunk_cache(int ncid, int p2, size_t* p3, size_t* p4, float* p5)
     return THROW(ret);
 }
 
-#endif /* USE_NETCDF4 */
