@@ -1,5 +1,5 @@
 /*********************************************************************
- *   Copyright 2016, UCAR/Unidata
+ *   Copyright 2018, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
  *********************************************************************/
 
@@ -26,7 +26,7 @@ If the sort is NCD4_NULL, then that means it is
 irrelevant because that keyword will never be
 searched for in this table.
 */
-struct KEYWORDINFO {
+static const struct KEYWORDINFO {
     char* tag; /* The xml tag e.g. <tag...> */
     NCD4sort sort; /* What kind of node are we building */
     nc_type subsort; /* discriminator */
@@ -63,7 +63,7 @@ struct KEYWORDINFO {
 };
 typedef struct KEYWORDINFO KEYWORDINFO;
 
-static struct ATOMICTYPEINFO {
+static const struct ATOMICTYPEINFO {
     char* name; nc_type type; size_t size;
 } atomictypeinfo[] = {
 /* Keep in sorted order for binary search */
@@ -106,7 +106,6 @@ extern const char** ezxml_all_attr(ezxml_t xml, int* countp);
 
 static int addOrigType(NCD4parser*, NCD4node* src, NCD4node* dst, const char* tag);
 static int defineAtomicTypes(NCD4parser*);
-static int defineBytestringType(NCD4parser*);
 static void classify(NCD4node* container, NCD4node* node);
 static int convertString(union ATOMICS*, NCD4node* type, const char* s);
 static int downConvert(union ATOMICS*, NCD4node* type);
@@ -114,7 +113,7 @@ static int fillgroup(NCD4parser*, NCD4node* group, ezxml_t xml);
 static NCD4node* getOpaque(NCD4parser*, ezxml_t varxml, NCD4node* group);
 static int getValueStrings(NCD4parser*, NCD4node*, ezxml_t xattr, NClist*);
 static int isReserved(const char* name);
-static KEYWORDINFO* keyword(const char* name);
+static const KEYWORDINFO* keyword(const char* name);
 static NCD4node* lookupAtomictype(NCD4parser*, const char* name);
 static NCD4node* lookFor(NClist* elems, const char* name, NCD4sort sort);
 static NCD4node* lookupFQN(NCD4parser*, const char* sfqn, NCD4sort);
@@ -143,6 +142,9 @@ static void record(NCD4parser*, NCD4node* node);
 static int splitOrigType(NCD4parser*, const char* fqn, NCD4node* var);
 static void track(NCD4parser*, NCD4node* node);
 static int traverse(NCD4parser*, ezxml_t dom);
+#ifndef FIXEDOPAQUE
+static int defineBytestringType(NCD4parser*);
+#endif
 
 /***************************************************/
 /* API */
@@ -182,11 +184,19 @@ done:
 static void
 reclaimParser(NCD4parser* parser)
 {
+    int i,len;
     if(parser == NULL) return;
-    nclistfree(parser->atomictypes);
     nclistfree(parser->types);
     nclistfree(parser->dims);
     nclistfree(parser->vars);
+    /* Reclaim unused atomic type nodes */
+    len = nclistlength(parser->atomictypes);    
+    for(i=0;i<len;i++) {
+	if(parser->used[i])
+	    reclaimNode((NCD4node*)nclistget(parser->atomictypes,i));
+    }
+    nclistfree(parser->atomictypes);
+    nullfree(parser->used);
     free (parser);
 }
 
@@ -335,7 +345,7 @@ parseVariables(NCD4parser* parser, NCD4node* group, ezxml_t xml)
     ezxml_t x;
     for(x=xml->child;x != NULL;x=x->ordered) {
 	NCD4node* node = NULL;
-	KEYWORDINFO* info = keyword(x->name);
+	const KEYWORDINFO* info = keyword(x->name);
 	if(info == NULL)
 	    FAIL(NC_ETRANSLATION,"Unexpected node type: %s",x->name);
 	/* Check if we need to process this node */
@@ -353,7 +363,7 @@ parseVariable(NCD4parser* parser, NCD4node* container, ezxml_t xml, NCD4node** n
 {
     int ret = NC_NOERR;
     NCD4node* node = NULL;
-    KEYWORDINFO* info = keyword(xml->name);
+    const KEYWORDINFO* info = keyword(xml->name);
 
     switch (info->subsort) {
     case NC_STRUCT:
@@ -441,7 +451,7 @@ parseFields(NCD4parser* parser, NCD4node* container, ezxml_t xml)
     ezxml_t x;
     for(x=xml->child;x != NULL;x=x->ordered) {
 	NCD4node* node = NULL;
-        KEYWORDINFO* info = keyword(x->name);
+        const KEYWORDINFO* info = keyword(x->name);
 	if(!ISVAR(info->sort)) continue; /* not a field */
 	ret = parseVariable(parser,container,x,&node);
 	if(ret) goto done;
@@ -461,7 +471,7 @@ parseVlenField(NCD4parser* parser, NCD4node* container, ezxml_t xml, NCD4node**
     NCD4node* field = NULL;
     ezxml_t x;
     for(x=xml->child;x != NULL;x=x->ordered) {
-        KEYWORDINFO* info = keyword(x->name);
+        const KEYWORDINFO* info = keyword(x->name);
 	if(!ISVAR(info->sort)) continue; /* not a field */
 	if(field != NULL)
 	    {ret = NC_EBADTYPE; goto done;}
@@ -492,7 +502,7 @@ parseSequence(NCD4parser* parser, NCD4node* container, ezxml_t xml, NCD4node** n
 	2. a vlen type whose basetype is #1
 	3. a variable whose basetype is #2.
 	If we can infer that the sequence was riginally produced
-	from a netcdf-4 vlen, then we can avoid createing #1.
+	from a netcdf-4 vlen, then we can avoid creating #1.
 	Naming is as follows. Assume the var name is V
 	and the NCD4_makeName of the var is V1..._Vn.
 	1. var name is V.
@@ -525,7 +535,7 @@ parseSequence(NCD4parser* parser, NCD4node* container, ezxml_t xml, NCD4node** n
 	vlentype->basetype = var->basetype;
 	/* Use name <fqnname>_t */
 	strncpy(name,fqnname,sizeof(name));
-	strlcat(name,"_t",sizeof(name));
+	strncat(name,"_t", sizeof(name) - strlen(name) - 1);
         SETNAME(vlentype,name);
         /* Set the basetype */
         var->basetype = vlentype;
@@ -540,7 +550,7 @@ parseSequence(NCD4parser* parser, NCD4node* container, ezxml_t xml, NCD4node** n
         classify(group,structtype);
 	/* Use name <fqnname>_base */
 	strncpy(name,fqnname,sizeof(name));
-	strlcat(name,"_base",sizeof(name));
+	strncat(name,"_base", sizeof(name) - strlen(name) - 1);
         SETNAME(structtype,name);
         /* Parse Fields into type */
         if((ret = parseFields(parser,structtype,xml))) goto done;
@@ -549,7 +559,7 @@ parseSequence(NCD4parser* parser, NCD4node* container, ezxml_t xml, NCD4node** n
         classify(group,vlentype);
 	/* Use name <xname>_t */
 	strncpy(name,fqnname,sizeof(name));
-	strlcat(name,"_t",sizeof(name));
+	strncat(name,"_t", sizeof(name) - strlen(name) - 1);
         SETNAME(vlentype,name);
 	vlentype->basetype = structtype;
         /* Set the basetype */
@@ -603,7 +613,7 @@ parseAtomicVar(NCD4parser* parser, NCD4node* container, ezxml_t xml, NCD4node**
     NCD4node* node = NULL;
     NCD4node* base = NULL;
     const char* typename;
-    KEYWORDINFO* info;
+    const KEYWORDINFO* info;
     NCD4node* group;
 
     /* Check for aliases */
@@ -742,12 +752,7 @@ parseAttributes(NCD4parser* parser, NCD4node* container, ezxml_t xml)
 	}
 
 	if((ret=makeNode(parser,container,x,NCD4_ATTR,NC_NULL,&attr))) goto done;
-	/* HACK: If the attribute is _FillValue, then force the use of the
-           container's type as the attribute type */
-	if(strcmp(attr->name,"_FillValue") == 0)
-	    basetype = container->basetype;
-	else
-	    basetype = lookupFQN(parser,type,NCD4_TYPE);
+	basetype = lookupFQN(parser,type,NCD4_TYPE);
 	if(basetype == NULL)
 	    FAIL(NC_EBADTYPE,"Unknown <Attribute> type: ",type);
 	if(basetype->subsort == NC_NAT && basetype->subsort != NC_ENUM)
@@ -804,22 +809,31 @@ getOpaque(NCD4parser* parser, ezxml_t varxml, NCD4node* group)
     NCD4node* opaquetype = NULL;
     const char* xattr;
 
+#ifndef FIXEDOPAQUE
+    len = 0;
+#else
+    len = parser->metadata->controller->controls.opaquesize;
+#endif
     if(parser->metadata->controller->controls.translation == NCD4_TRANSNC4) {
         /* See if this var has UCARTAGOPAQUE attribute */
         xattr = ezxml_attr(varxml,UCARTAGOPAQUE);
         if(xattr != NULL) {
-            if((ret = parseLL(xattr,&len)) || (len < 0))
+	    long long tmp = 0;
+            if((ret = parseLL(xattr,&tmp)) || (tmp < 0))
 	        FAIL(NC_EINVAL,"Illegal opaque len: %s",xattr);
-        } else
-	    len = 0;
-    } else
-	len = 0;
-    if(len == 0) { /* Need to use _bytestring */
+	    len = tmp;
+        }
+    }
+#ifndef FIXEDOPAQUE
+    if(len == 0) {
+        /* Need to use _bytestring */
 	if((ret=defineBytestringType(parser)))
   	    goto done;
 	assert(parser->metadata->_bytestring != NULL);
 	opaquetype = parser->metadata->_bytestring;
-    } else {//(len > 0)
+    } else
+#endif
+    { /*(len > 0) || FIXEDOPAQUE */
         /* Try to locate existing opaque type with this length */
         for(i=0;i<nclistlength(parser->types); i++) {
 	    NCD4node* op = (NCD4node*)nclistget(parser->types,i);
@@ -1127,14 +1141,14 @@ done:
     return (ret == NC_NOERR ? match : NULL);
 }
 
-static KEYWORDINFO*
+static const KEYWORDINFO*
 keyword(const char* name)
 {
     int n = sizeof(keywordmap)/sizeof(KEYWORDINFO);
     int L = 0;
     int R = (n - 1);
     int m, cmp;
-    struct KEYWORDINFO* p;
+    const struct KEYWORDINFO* p;
     for(;;) {
 	if(L > R) break;
         m = (L + R) / 2;
@@ -1149,6 +1163,7 @@ keyword(const char* name)
     return NULL;
 }
 
+#ifndef FIXEDOPAQUE
 static int
 defineBytestringType(NCD4parser* parser)
 {
@@ -1168,13 +1183,14 @@ defineBytestringType(NCD4parser* parser)
 done:
     return THROW(ret);
 }
+#endif
 
 static int
 defineAtomicTypes(NCD4parser* parser)
 {
     int ret = NC_NOERR;
     NCD4node* node;
-    struct ATOMICTYPEINFO* ati;
+    const struct ATOMICTYPEINFO* ati;
 
     parser->atomictypes = nclistnew();
     if(parser->atomictypes == NULL)
@@ -1186,6 +1202,8 @@ defineAtomicTypes(NCD4parser* parser)
 	record(parser,node);
 	PUSH(parser->atomictypes,node);
     }
+    parser->used = (char*)calloc(1,nclistlength(parser->atomictypes));
+    if(parser->used == NULL) {ret = NC_ENOMEM; goto done;}
 
 done:
     return THROW(ret);
