@@ -1,5 +1,5 @@
 /*********************************************************************
- *   Copyright 1993, UCAR/Unidata
+ *   Copyright 2018, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
  *********************************************************************/
 
@@ -30,12 +30,9 @@ static char* constrainableprotocols[] = {"http", "https",NULL};
 
 static int ncd4initialized = 0;
 
-static NC_Dispatch NCD4_dispatch_base;
+static const NC_Dispatch NCD4_dispatch_base;
 
-NC_Dispatch* NCD4_dispatch_table = NULL; /* moved here from ddispatch.c */
-
-/* Collect global state info in one place */
-NCD4globalstate* NCD4_globalstate = NULL;
+const NC_Dispatch* NCD4_dispatch_table = NULL; /* moved here from ddispatch.c */
 
 /* Forward */
 static int globalinit(void);
@@ -57,6 +54,7 @@ NCD4_initialize(void)
     globalinit();
     /* Load rc file */
     NC_rcload();    
+
     return THROW(NC_NOERR);
 }
 
@@ -87,8 +85,7 @@ NCD4_sync(int ncid)
 static int
 NCD4_create(const char *path, int cmode,
            size_t initialsz, int basepe, size_t *chunksizehintp,
-	   int use_parallel, void* mpidata,
-           NC_Dispatch* dispatch, NC* ncp)
+           void* mpidata, const NC_Dispatch *dispatch, int ncid)
 {
    return THROW(NC_EPERM);
 }
@@ -120,12 +117,6 @@ NCD4_set_fill(int ncid, int fillmode, int* old_modep)
     return (NC_EPERM);
 }
 
-static int
-NCD4_set_base_pe(int ncid, int pe)
-{
-    return (NC_EPERM);
-}
-
 static int
 NCD4_def_dim(int ncid, const char* name, size_t len, int* idp)
 {
@@ -256,18 +247,6 @@ Following functions basically return the netcdf-4 value WRT to the nc4id.
 However, it is necessary to modify the grpid(ncid) to point to the substrate.
 */
 
-static int
-NCD4_inq_base_pe(int ncid, int* pe)
-{
-    NC* ncp;
-    int ret = NC_NOERR;
-    int substrateid;
-    if((ret = NC_check_id(ncid, (NC**)&ncp)) != NC_NOERR) return (ret);
-    substrateid = makenc4id(ncp,ncid);
-    ret = nc_inq_base_pe(substrateid, pe);
-    return (ret);
-}
-
 static int
 NCD4_inq_format(int ncid, int* formatp)
 {
@@ -467,8 +446,6 @@ NCD4_var_par_access(int ncid, int p2, int p3)
 }
 
 
-#ifdef USE_NETCDF4
-
 static int
 NCD4_inq_ncid(int ncid, const char* name, int* grp_ncid)
 {
@@ -730,8 +707,6 @@ NCD4_get_var_chunk_cache(int ncid, int p2, size_t* p3, size_t* p4, float* p5)
     return (ret);
 }
 
-#endif // USE_NETCDF4
-
 /**************************************************/
 /*
 Following functions are overridden to handle 
@@ -795,17 +770,12 @@ static int
 globalinit(void)
 {
     int stat = NC_NOERR;
-    {
-	CURLcode cstat = curl_global_init(CURL_GLOBAL_DEFAULT);
-	if(cstat != CURLE_OK)
-	    fprintf(stderr,"curl_global_init failed!\n");
-    }
     return stat;
 }
 
 /**************************************************/
 
-static NC_Dispatch NCD4_dispatch_base = {
+static const NC_Dispatch NCD4_dispatch_base = {
 
 NC_FORMATX_DAP4,
 
@@ -818,8 +788,6 @@ NCD4_sync,
 NCD4_abort,
 NCD4_close,
 NCD4_set_fill,
-NCD4_inq_base_pe,
-NCD4_set_base_pe,
 NCD4_inq_format,
 NCD4_inq_format_extended, /*inq_format_extended*/
 
@@ -853,8 +821,8 @@ NCDEFAULT_put_varm,
 NCD4_inq_var_all,
 
 NCD4_var_par_access,
+NCD4_def_var_fill,
 
-#ifdef USE_NETCDF4
 NCD4_show_metadata,
 NCD4_inq_unlimdims,
 NCD4_inq_ncid,
@@ -888,14 +856,11 @@ NCD4_def_opaque,
 NCD4_def_var_deflate,
 NCD4_def_var_fletcher32,
 NCD4_def_var_chunking,
-NCD4_def_var_fill,
 NCD4_def_var_endian,
 NCD4_def_var_filter,
 NCD4_set_var_chunk_cache,
 NCD4_get_var_chunk_cache,
 
-#endif /*USE_NETCDF4*/
-
 };
 
 
