@@ -1,5 +1,5 @@
 /*
- *	Copyright 1996, University Corporation for Atmospheric Research
+ *	Copyright 2018, University Corporation for Atmospheric Research
  *      See netcdf/COPYRIGHT file for copying and redistribution conditions.
  */
 /* $Id: var.c,v 1.144 2010/05/30 00:50:35 russ Exp $ */
@@ -16,6 +16,7 @@
 #include "ncx.h"
 #include "rnd.h"
 #include "ncutf8.h"
+#include "nc3dispatch.h"
 
 #ifndef OFF_T_MAX
 #if 0
@@ -177,7 +178,7 @@ dup_NC_var(const NC_var *rvarp)
 	(void) memcpy(varp->shape, rvarp->shape,
 			 rvarp->ndims * sizeof(size_t));
 	(void) memcpy(varp->dsizes, rvarp->dsizes,
-			 rvarp->ndims * sizeof(size_t));
+			 rvarp->ndims * sizeof(off_t));
 	varp->xsz = rvarp->xsz;
 	varp->len = rvarp->len;
 	varp->begin = rvarp->begin;
@@ -229,7 +230,7 @@ free_NC_vararrayV(NC_vararray *ncap)
 	if(ncap->nalloc == 0)
 		return;
 
-	NC_hashmapDelete(ncap->hashmap);
+	NC_hashmapfree(ncap->hashmap);
 	ncap->hashmap = NULL;
 
 	assert(ncap->value != NULL);
@@ -309,7 +310,7 @@ incr_NC_vararray(NC_vararray *ncap, NC_var *newelemp)
 		ncap->value = vp;
 		ncap->nalloc = NC_ARRAY_GROWBY;
 
-		ncap->hashmap = NC_hashmapCreate(0);
+		ncap->hashmap = NC_hashmapnew(0);
 	}
 	else if(ncap->nelems +1 > ncap->nalloc)
 	{
@@ -323,7 +324,7 @@ incr_NC_vararray(NC_vararray *ncap, NC_var *newelemp)
 
 	if(newelemp != NULL)
 	{
-		NC_hashmapAddVar(ncap, (long)ncap->nelems, newelemp->name->cp);
+		NC_hashmapadd(ncap->hashmap, (uintptr_t)ncap->nelems, newelemp->name->cp, strlen(newelemp->name->cp));
 		ncap->value[ncap->nelems] = newelemp;
 		ncap->nelems++;
 	}
@@ -352,35 +353,33 @@ elem_NC_vararray(const NC_vararray *ncap, size_t elem)
  * Step thru NC_VARIABLE array, seeking match on name.
  * Return varid or -1 on not found.
  * *varpp is set to the appropriate NC_var.
- * Formerly (sort of)
-NC_hvarid
+ * Formerly (sort of) NC_hvarid
  */
 int
 NC_findvar(const NC_vararray *ncap, const char *uname, NC_var **varpp)
 {
-	int hash_var_id;
-	char *name;
-	int stat;
+	int hash_var_id = -1;
+	uintptr_t data;
+	char *name = NULL;
 
 	assert(ncap != NULL);
 
 	if(ncap->nelems == 0)
-		return -1;
-
+	    goto done;
 
 	/* normalized version of uname */
-        stat = nc_utf8_normalize((const unsigned char *)uname,(unsigned char **)&name);
-        if(stat != NC_NOERR)
-	    return stat;
+        if(nc_utf8_normalize((const unsigned char *)uname,(unsigned char **)&name))
+	    goto done;
 
-	hash_var_id = (int)NC_hashmapGetVar(ncap, name);
-	free(name);
-	if (hash_var_id >= 0) {
-	  if (varpp != NULL)
-	    *varpp = ncap->value[hash_var_id];
-	  return(hash_var_id); /* Normal return */
-	}
-	return(-1); /* not found */
+	if(NC_hashmapget(ncap->hashmap, name, strlen(name), &data) == 0)
+	    goto done;
+
+	hash_var_id = (int)data;
+        if (varpp != NULL)
+	  *varpp = ncap->value[hash_var_id];
+done:
+	if(name != NULL) free(name);
+	return(hash_var_id); /* Normal return */
 }
 
 /*
@@ -484,29 +483,18 @@ NC_var_shape(NC_var *varp, const NC_dimarray *dims)
 
 
 out :
-    if( varp->xsz <= (X_UINT_MAX - 1) / product ) /* if integer multiply will not overflow */
-	{
-	        varp->len = product * varp->xsz;
-		switch(varp->type) {
-		case NC_BYTE :
-		case NC_CHAR :
-		case NC_UBYTE :
-		case NC_SHORT :
-		case NC_USHORT :
-		        if( varp->len%4 != 0 )
-			{
-			        varp->len += 4 - varp->len%4; /* round up */
-		/*		*dsp += 4 - *dsp%4; */
-		    }
-		    break;
-		default:
-			/* already aligned */
-			break;
-		}
-        } else
-	{	/* OK for last var to be "too big", indicated by this special len */
-	        varp->len = X_UINT_MAX;
-        }
+
+    /*
+     * For CDF-1 and CDF-2 formats, the total number of array elements
+     * cannot exceed 2^32, unless this variable is the last fixed-size
+     * variable, there is no record variable, and the file starting
+     * offset of this variable is less than 2GiB.
+     * This will be checked in NC_check_vlens() during NC_endef()
+     */
+    varp->len = product * varp->xsz;
+    if (varp->len % 4 > 0)
+        varp->len += 4 - varp->len % 4; /* round up */
+
 #if 0
 	arrayp("\tshape", varp->ndims, varp->shape);
 	arrayp("\tdsizes", varp->ndims, varp->dsizes);
@@ -522,16 +510,15 @@ out :
  * systems with LFS it should be 2^32 - 4.
  */
 int
-NC_check_vlen(NC_var *varp, size_t vlen_max) {
-    size_t prod=varp->xsz;	/* product of xsz and dimensions so far */
-
+NC_check_vlen(NC_var *varp, long long vlen_max) {
     int ii;
+    long long prod=varp->xsz;	/* product of xsz and dimensions so far */
 
     assert(varp != NULL);
     for(ii = IS_RECVAR(varp) ? 1 : 0; ii < varp->ndims; ii++) {
       if(!varp->shape)
         return 0; /* Shape is undefined/NULL. */
-      if (varp->shape[ii] > vlen_max / prod) {
+      if ((long long)varp->shape[ii] > vlen_max / prod) {
         return 0;		/* size in bytes won't fit in a 32-bit int */
       }
       prod *= varp->shape[ii];
@@ -637,6 +624,13 @@ NC3_def_var( int ncid, const char *name, nc_type type,
 
 	if(varidp != NULL)
 		*varidp = (int)ncp->vars.nelems -1; /* varid */
+
+	/* set the variable's fill mode */
+	if (NC_dofill(ncp))
+		varp->no_fill = 0;
+	else
+		varp->no_fill = 1;
+
 	return NC_NOERR;
 }
 
@@ -673,7 +667,9 @@ NC3_inq_var(int ncid,
 	nc_type *typep,
 	int *ndimsp,
 	int *dimids,
-	int *nattsp)
+	int *nattsp,
+	int *no_fillp,
+	void *fill_valuep)
 {
 	int status;
 	NC *nc;
@@ -714,19 +710,114 @@ NC3_inq_var(int ncid,
 		*nattsp = (int) varp->attrs.nelems;
 	}
 
+	if (no_fillp != NULL) *no_fillp = varp->no_fill;
+
+	if (fill_valuep != NULL) {
+		status = nc_get_att(ncid, varid, _FillValue, fill_valuep);
+		if (status != NC_NOERR && status != NC_ENOTATT)
+			return status;
+		if (status == NC_ENOTATT) {
+			status = NC3_inq_default_fill_value(varp->type, fill_valuep);
+			if (status != NC_NOERR) return status;
+		}
+	}
+
 	return NC_NOERR;
 }
 
 int
 NC3_rename_var(int ncid, int varid, const char *unewname)
+{
+	int status = NC_NOERR;
+	NC *nc;
+	NC3_INFO* ncp;
+	uintptr_t intdata;
+	NC_var *varp;
+	NC_string *old, *newStr;
+	int other;
+	char *newname = NULL;	/* normalized */
+
+	status = NC_check_id(ncid, &nc);
+	if(status != NC_NOERR)
+	    goto done;
+	ncp = NC3_DATA(nc);
+
+	if(NC_readonly(ncp))
+	    {status = NC_EPERM; goto done;}
+
+	status = NC_check_name(unewname);
+	if(status != NC_NOERR)
+	    goto done;
+
+	/* check for name in use */
+	other = NC_findvar(&ncp->vars, unewname, &varp);
+	if(other != -1)
+	    {status = NC_ENAMEINUSE; goto done;}
+
+	status = NC_lookupvar(ncp, varid, &varp);
+	if(status != NC_NOERR)
+	    goto done; /* invalid varid */
+
+	old = varp->name;
+        status = nc_utf8_normalize((const unsigned char *)unewname,(unsigned char **)&newname);
+        if(status != NC_NOERR)
+	    goto done;
+	if(NC_indef(ncp))
+	{
+		/* Remove old name from hashmap; add new... */
+	        /* WARNING: strlen(NC_string.cp) may be less than NC_string.nchars */
+		NC_hashmapremove(ncp->vars.hashmap,old->cp,strlen(old->cp),NULL);
+		newStr = new_NC_string(strlen(newname),newname);
+		if(newStr == NULL)
+		    {status = NC_ENOMEM; goto done;}
+		varp->name = newStr;
+		intdata = (uintptr_t)varid;
+		NC_hashmapadd(ncp->vars.hashmap, intdata, varp->name->cp, strlen(varp->name->cp));
+		free_NC_string(old);
+		goto done;
+	}
+
+	/* else, not in define mode */
+	/* If new name is longer than old, then complain,
+           but otherwise, no change (test is same as set_NC_string)*/
+	if(varp->name->nchars < strlen(newname))
+	    {status = NC_ENOTINDEFINE; goto done;}
+
+	/* WARNING: strlen(NC_string.cp) may be less than NC_string.nchars */
+	/* Remove old name from hashmap; add new... */
+        NC_hashmapremove(ncp->vars.hashmap,old->cp,strlen(old->cp),NULL);
+
+	/* WARNING: strlen(NC_string.cp) may be less than NC_string.nchars */
+	status = set_NC_string(varp->name, newname);
+	if(status != NC_NOERR)
+		goto done;
+
+	intdata = (uintptr_t)varid;
+	NC_hashmapadd(ncp->vars.hashmap, intdata, varp->name->cp, strlen(varp->name->cp));
+
+	set_NC_hdirty(ncp);
+
+	if(NC_doHsync(ncp))
+	{
+		status = NC_sync(ncp);
+		if(status != NC_NOERR)
+			goto done;
+	}
+done:
+	if(newname) free(newname);
+	return status;
+}
+
+int
+NC3_def_var_fill(int ncid,
+	int varid,
+	int no_fill,
+	const void *fill_value)
 {
 	int status;
 	NC *nc;
 	NC3_INFO* ncp;
 	NC_var *varp;
-	NC_string *old, *newStr;
-	int other;
-	char *newname;		/* normalized */
 
 	status = NC_check_id(ncid, &nc);
 	if(status != NC_NOERR)
@@ -738,63 +829,31 @@ NC3_rename_var(int ncid, int varid, const char *unewname)
 		return NC_EPERM;
 	}
 
-	status = NC_check_name(unewname);
-	if(status != NC_NOERR)
-		return status;
-
-	/* check for name in use */
-	other = NC_findvar(&ncp->vars, unewname, &varp);
-	if(other != -1)
+	if(!NC_indef(ncp))
 	{
-		return NC_ENAMEINUSE;
+		return NC_ENOTINDEFINE;
 	}
 
-	status = NC_lookupvar(ncp, varid, &varp);
-	if(status != NC_NOERR)
-	{
-		/* invalid varid */
-		return status;
-	}
+	varp = elem_NC_vararray(&ncp->vars, (size_t)varid);
+	if(varp == NULL)
+		return NC_ENOTVAR;
 
+	if (no_fill)
+		varp->no_fill = 1;
+	else
+		varp->no_fill = 0;
 
-	old = varp->name;
-        status = nc_utf8_normalize((const unsigned char *)unewname,(unsigned char **)&newname);
-        if(status != NC_NOERR)
-	    return status;
-	if(NC_indef(ncp))
-	{
-		/* Remove old name from hashmap; add new... */
-		NC_hashmapRemoveVar(&ncp->vars, old->cp);
+	/* Are we setting a fill value? */
+	if (fill_value != NULL && !varp->no_fill) {
 
-		newStr = new_NC_string(strlen(newname),newname);
-		free(newname);
-		if(newStr == NULL)
-			return(-1);
-		varp->name = newStr;
-		NC_hashmapAddVar(&ncp->vars, varid, newStr->cp);
-		free_NC_string(old);
-
-		return NC_NOERR;
-	}
-
-	/* else, not in define mode */
-	/* Remove old name from hashmap; add new... */
-	NC_hashmapRemoveVar(&ncp->vars, old->cp);
-
-	status = set_NC_string(varp->name, newname);
-	free(newname);
-	if(status != NC_NOERR)
-		return status;
-
-	NC_hashmapAddVar(&ncp->vars, varid, varp->name->cp);
-
-	set_NC_hdirty(ncp);
-
-	if(NC_doHsync(ncp))
-	{
-		status = NC_sync(ncp);
-		if(status != NC_NOERR)
+		/* If there's a _FillValue attribute, delete it. */
+		status = NC3_del_att(ncid, varid, _FillValue);
+		if (status != NC_NOERR && status != NC_ENOTATT)
 			return status;
+
+		/* Create/overwrite attribute _FillValue */
+		status = NC3_put_att(ncid, varid, _FillValue, varp->type, 1, fill_value, varp->type);
+		if (status != NC_NOERR) return status;
 	}
 
 	return NC_NOERR;
