@@ -1,4 +1,4 @@
-/* This is part of the netCDF package. Copyright 2005-2007 University
+/* This is part of the netCDF package. Copyright 2005-2018 University
    Corporation for Atmospheric Research/Unidata. See COPYRIGHT file
    for conditions of use.
 
@@ -99,7 +99,7 @@ tst_att_ordering(int cmode)
 
    /* Create a file with some global atts. */
 #ifdef TEST_PNETCDF
-   if (nc_create_par(FILE_NAME, cmode|NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
+   if (nc_create_par(FILE_NAME, cmode, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
 #else
    if (nc_create(FILE_NAME, cmode, &ncid)) ERR;
 #endif
@@ -115,7 +115,7 @@ tst_att_ordering(int cmode)
 
    /* Reopen the file and check the order. */
 #ifdef TEST_PNETCDF
-   if (nc_open_par(FILE_NAME, NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
+   if (nc_open_par(FILE_NAME, 0, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
 #else
    if (nc_open(FILE_NAME, 0, &ncid)) ERR;
 #endif
@@ -140,14 +140,6 @@ main(int argc, char **argv)
     float float_in[ATT_LEN], float_out[ATT_LEN] = {-0.5, 0.25, 0.125};
     double double_in[ATT_LEN], double_out[ATT_LEN] = {-0.25, .5, 0.125};
     long long longlong_in[ATT_LEN] = {-1LL, -1LL, -1LL};
-#ifdef USE_NETCDF4
-    long long_in[ATT_LEN];
-    unsigned short ushort_in[ATT_LEN], ushort_out[ATT_LEN] = {0, 128, NC_MAX_USHORT};
-    unsigned int uint_in[ATT_LEN], uint_out[ATT_LEN] = {0, 128, NC_MAX_UINT};
-    long long longlong_out[ATT_LEN] = {-3123456789LL, 128LL, 3123456789LL};
-    unsigned long long ulonglong_in[ATT_LEN] = {NC_MAX_UINT64, NC_MAX_UINT64, NC_MAX_UINT64};
-    unsigned long long ulonglong_out[ATT_LEN] = {0LL, 128LL, 3123456789LL};
-#endif
 
 #ifdef TEST_PNETCDF
    MPI_Init(&argc, &argv);
@@ -166,7 +158,7 @@ main(int argc, char **argv)
 
       /* Create a file with some global atts. */
 #ifdef TEST_PNETCDF
-      if (nc_create_par(FILE_NAME, NC_CLOBBER|NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
+      if (nc_create_par(FILE_NAME, NC_CLOBBER, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
 #else
       if (nc_create(FILE_NAME, NC_CLOBBER, &ncid)) ERR;
 #endif
@@ -176,7 +168,7 @@ main(int argc, char **argv)
 
       /* Reopen the file and check the order. */
 #ifdef TEST_PNETCDF
-      if (nc_open_par(FILE_NAME, NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
+      if (nc_open_par(FILE_NAME, 0, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
 #else
       if (nc_open(FILE_NAME, 0, &ncid)) ERR;
 #endif
@@ -201,7 +193,7 @@ main(int argc, char **argv)
 
       /* This won't work, because classic files can't create these types. */
 #ifdef TEST_PNETCDF
-      if (nc_create_par(FILE_NAME, NC_CLOBBER|NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
+      if (nc_create_par(FILE_NAME, NC_CLOBBER, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
 #else
       if (nc_create(FILE_NAME, NC_CLOBBER, &ncid)) ERR;
 #endif
@@ -224,7 +216,7 @@ main(int argc, char **argv)
 
       /* Create a file with a global attribute of each type. */
 #ifdef TEST_PNETCDF
-      if (nc_create_par(FILE_NAME, NC_CLOBBER|NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
+      if (nc_create_par(FILE_NAME, NC_CLOBBER, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
 #else
       if (nc_create(FILE_NAME, NC_CLOBBER, &ncid)) ERR;
 #endif
@@ -238,7 +230,7 @@ main(int argc, char **argv)
 
       /* Open the file and check attributes. */
 #ifdef TEST_PNETCDF
-      if (nc_open_par(FILE_NAME, NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
+      if (nc_open_par(FILE_NAME, 0, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
 #else
       if (nc_open(FILE_NAME, 0, &ncid)) ERR;
 #endif
@@ -277,7 +269,7 @@ main(int argc, char **argv)
 
       /* Reopen the file and try different type conversions. */
 #ifdef TEST_PNETCDF
-      if (nc_open_par(FILE_NAME, NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
+      if (nc_open_par(FILE_NAME, 0, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
 #else
       if (nc_open(FILE_NAME, 0, &ncid)) ERR;
 #endif
@@ -431,7 +423,7 @@ main(int argc, char **argv)
 
       /* Create a file with a global attribute of each type of zero length. */
 #ifdef TEST_PNETCDF
-      if (nc_create_par(FILE_NAME, NC_CLOBBER|NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
+      if (nc_create_par(FILE_NAME, NC_CLOBBER, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
 #else
       if (nc_create(FILE_NAME, NC_CLOBBER, &ncid)) ERR;
 #endif
@@ -457,7 +449,7 @@ main(int argc, char **argv)
       nc_type xtype;
 
 #ifdef TEST_PNETCDF
-      if (nc_open_par(FILE_NAME, NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
+      if (nc_open_par(FILE_NAME, 0, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
 #else
       if (nc_open(FILE_NAME, 0, &ncid)) ERR;
 #endif
@@ -500,7 +492,7 @@ main(int argc, char **argv)
 
       /* Create a file with a global attribute of each type of zero length. */
 #ifdef TEST_PNETCDF
-      if (nc_create_par(FILE_NAME, NC_CLOBBER|NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
+      if (nc_create_par(FILE_NAME, NC_CLOBBER, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
 #else
       if (nc_create(FILE_NAME, NC_CLOBBER, &ncid)) ERR;
 #endif
@@ -517,7 +509,7 @@ main(int argc, char **argv)
       /* Make sure we can read all these zero-length atts added during a
        * redef. */
 #ifdef TEST_PNETCDF
-      if (nc_open_par(FILE_NAME, NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
+      if (nc_open_par(FILE_NAME, 0, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
 #else
       if (nc_open(FILE_NAME, 0, &ncid)) ERR;
 #endif
@@ -548,7 +540,7 @@ main(int argc, char **argv)
 
       /* Create a file with a global attribute. */
 #ifdef TEST_PNETCDF
-      if (nc_create_par(FILE_NAME, NC_CLOBBER|NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
+      if (nc_create_par(FILE_NAME, NC_CLOBBER, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
 #else
       if (nc_create(FILE_NAME, NC_CLOBBER, &ncid)) ERR;
 #endif
@@ -558,7 +550,7 @@ main(int argc, char **argv)
 
       /* Rename it. */
 #ifdef TEST_PNETCDF
-      if (nc_open_par(FILE_NAME, NC_WRITE|NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
+      if (nc_open_par(FILE_NAME, NC_WRITE, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
 #else
       if (nc_open(FILE_NAME, NC_WRITE, &ncid)) ERR;
 #endif
@@ -573,7 +565,7 @@ main(int argc, char **argv)
       if (nc_close(ncid)) ERR;
 
 #ifdef TEST_PNETCDF
-      if (nc_open_par(FILE_NAME, NC_WRITE|NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
+      if (nc_open_par(FILE_NAME, NC_WRITE, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
 #else
       if (nc_open(FILE_NAME, NC_WRITE, &ncid)) ERR;
 #endif
@@ -588,7 +580,7 @@ main(int argc, char **argv)
 
       /* Now delete the att. */
 #ifdef TEST_PNETCDF
-      if (nc_open_par(FILE_NAME, NC_WRITE|NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
+      if (nc_open_par(FILE_NAME, NC_WRITE, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
 #else
       if (nc_open(FILE_NAME, NC_WRITE, &ncid)) ERR;
 #endif
@@ -598,7 +590,7 @@ main(int argc, char **argv)
 
       /* Now create a file with a variable, which has an att. */
 #ifdef TEST_PNETCDF
-      if (nc_create_par(FILE_NAME, NC_CLOBBER|NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
+      if (nc_create_par(FILE_NAME, NC_CLOBBER, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
 #else
       if (nc_create(FILE_NAME, NC_CLOBBER, &ncid)) ERR;
 #endif
@@ -611,7 +603,7 @@ main(int argc, char **argv)
 
       /* Reopen the file and delete it. Make sure it's gone. */
 #ifdef TEST_PNETCDF
-      if (nc_open_par(FILE_NAME, NC_WRITE|NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
+      if (nc_open_par(FILE_NAME, NC_WRITE, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
 #else
       if (nc_open(FILE_NAME, NC_WRITE, &ncid)) ERR;
 #endif
@@ -622,7 +614,7 @@ main(int argc, char **argv)
       /* Reopen the file and readd the attribute. Enddef and redef,
        * and delete it, then check to make sure it's gone. */
 #ifdef TEST_PNETCDF
-      if (nc_open_par(FILE_NAME, NC_WRITE|NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
+      if (nc_open_par(FILE_NAME, NC_WRITE, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
 #else
       if (nc_open(FILE_NAME, NC_WRITE, &ncid)) ERR;
 #endif
@@ -648,7 +640,7 @@ main(int argc, char **argv)
 
       /* Create a file with several global attributes. */
 #ifdef TEST_PNETCDF
-      if (nc_create_par(FILE_NAME, NC_CLOBBER|NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
+      if (nc_create_par(FILE_NAME, NC_CLOBBER, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
 #else
       if (nc_create(FILE_NAME, NC_CLOBBER, &ncid)) ERR;
 #endif
@@ -658,7 +650,7 @@ main(int argc, char **argv)
 
       /* Open it and check the order. */
 #ifdef TEST_PNETCDF
-      if (nc_open_par(FILE_NAME, NC_WRITE|NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
+      if (nc_open_par(FILE_NAME, NC_WRITE, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
 #else
       if (nc_open(FILE_NAME, NC_WRITE, &ncid)) ERR;
 #endif
@@ -670,7 +662,7 @@ main(int argc, char **argv)
 
       /* Now create a file with a variable, which has two atts. */
 #ifdef TEST_PNETCDF
-      if (nc_create_par(FILE_NAME, NC_CLOBBER|NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
+      if (nc_create_par(FILE_NAME, NC_CLOBBER, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
 #else
       if (nc_create(FILE_NAME, NC_CLOBBER, &ncid)) ERR;
 #endif
@@ -683,7 +675,7 @@ main(int argc, char **argv)
 
       /* Reopen the file and check the order of the attributes on the var. */
 #ifdef TEST_PNETCDF
-      if (nc_open_par(FILE_NAME, NC_WRITE|NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
+      if (nc_open_par(FILE_NAME, NC_WRITE, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
 #else
       if (nc_open(FILE_NAME, NC_WRITE, &ncid)) ERR;
 #endif
@@ -712,7 +704,7 @@ main(int argc, char **argv)
 
       /* Create a file with one var, and attach three atts to it. */
 #ifdef TEST_PNETCDF
-      if (nc_create_par(FILE_NAME, NC_CLOBBER|NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
+      if (nc_create_par(FILE_NAME, NC_CLOBBER, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
 #else
       if (nc_create(FILE_NAME, NC_CLOBBER, &ncid)) ERR;
 #endif
@@ -724,7 +716,7 @@ main(int argc, char **argv)
 
       /* Reopen the file and check. */
 #ifdef TEST_PNETCDF
-      if (nc_open_par(FILE_NAME, NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
+      if (nc_open_par(FILE_NAME, 0, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
 #else
       if (nc_open(FILE_NAME, 0, &ncid)) ERR;
 #endif
@@ -760,7 +752,7 @@ main(int argc, char **argv)
       int ncid, att = 1;
 
 #ifdef TEST_PNETCDF
-      if (nc_create_par(FILE_NAME, NC_CLOBBER|NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
+      if (nc_create_par(FILE_NAME, NC_CLOBBER, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
 #else
       if (nc_create(FILE_NAME, NC_CLOBBER, &ncid)) ERR;
 #endif
@@ -773,7 +765,7 @@ main(int argc, char **argv)
       if (nc_close(ncid)) ERR;
 
 #ifdef TEST_PNETCDF
-      if (nc_open_par(FILE_NAME, NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
+      if (nc_open_par(FILE_NAME, 0, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
 #else
       if (nc_open(FILE_NAME, 0, &ncid)) ERR;
 #endif
@@ -797,7 +789,7 @@ main(int argc, char **argv)
 
       /* Create a file with a global attribute of each type. */
 #ifdef TEST_PNETCDF
-      if (nc_create_par(FILE_NAME, NC_CLOBBER|NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
+      if (nc_create_par(FILE_NAME, NC_CLOBBER, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid)) ERR;
 #else
       if (nc_create(FILE_NAME, NC_CLOBBER, &ncid)) ERR;
 #endif
@@ -810,7 +802,7 @@ main(int argc, char **argv)
 
       /* Create another file and copy all the attributes. */
 #ifdef TEST_PNETCDF
-      if (nc_create_par(FILE_NAME2, NC_CLOBBER|NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid2)) ERR;
+      if (nc_create_par(FILE_NAME2, NC_CLOBBER, MPI_COMM_WORLD, MPI_INFO_NULL,&ncid2)) ERR;
 #else
       if (nc_create(FILE_NAME2, NC_CLOBBER, &ncid2)) ERR;
 #endif
@@ -827,7 +819,7 @@ main(int argc, char **argv)
 
       /* Open the file and check attributes. */
 #ifdef TEST_PNETCDF
-      if (nc_open_par(FILE_NAME2, NC_PNETCDF, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
+      if (nc_open_par(FILE_NAME2, 0, MPI_COMM_WORLD, MPI_INFO_NULL, &ncid)) ERR;
 #else
       if (nc_open(FILE_NAME2, 0, &ncid)) ERR;
 #endif
