@@ -1,6 +1,6 @@
 
 /*
- * Copyright 1998-2015 University Corporation for Atmospheric Research/Unidata
+ * Copyright 1998-2018 University Corporation for Atmospheric Research/Unidata
  *  See the LICENSE file for more information.
  */
 
@@ -68,7 +68,7 @@ static const struct Test utf8poems[] = {
 "Τὴ γλῶσσα μοῦ ἔδωσαν ἑλληνικὴτὸ σπίτι φτωχικὸ στὶς ἀμμουδιὲς τοῦ Ὁμήρου. Μονάχη ἔγνοια ἡ γλῶσσα μου στὶς ἀμμουδιὲς τοῦ Ὁμήρου.ἀπὸ τὸ Ἄξιον ἐστί τοῦ Ὀδυσσέα Ἐλύτη"
 },
 {0,"2.6",
-"Russion",
+"Russian",
 "На берегу пустынных волнСтоял он, дум великих полн,И вдаль глядел. Пред ним широкоРека неслася; бедный чёлнПо ней стремился одиноко.По мшистым, топким берегамЧернели избы здесь и там,Приют убогого чухонца;И лес, неведомый лучамВ тумане спрятанного солнца, Кругом шумел."
 },
 {0,"2.7",
