@@ -1,5 +1,5 @@
 /* This is part of the netCDF package.
-   Copyright 2005 University Corporation for Atmospheric Research/Unidata
+   Copyright 2018 University Corporation for Atmospheric Research/Unidata
     See COPYRIGHT file for conditions of use.
 
    Test netcdf-4 variables.
@@ -69,11 +69,19 @@ main(int argc, char **argv)
 #define FILE_NAME2 "tst_vars2_latefill.nc"
       printf("**** testing simple fill value attribute creation...");
       {
-         int status;
          int schar_data = 0;
          size_t index[1] = {0};
+         int expected_ret;
          int dimid;
 
+         /* Determined the expected result of setting fill value
+          * late. For historical reasons this is allowed for classic
+          * and 64-bit offset formats, but should never be done. */
+         if (cmode == 0 || cmode == NC_64BIT_OFFSET)
+            expected_ret = NC_NOERR;
+         else
+            expected_ret = NC_ELATEFILL;
+            
          /* Create a netcdf-4 file with one scalar var. Add fill
           * value. */
          if (nc_create(FILE_NAME2, cmode, &ncid)) ERR;
@@ -83,9 +91,8 @@ main(int argc, char **argv)
          if (nc_enddef(ncid)) ERR;
          if (nc_put_var1(ncid, varid, index, &schar_data)) ERR;
          if (nc_redef(ncid)) ERR;
-         status = nc_put_att_schar(ncid, varid, _FillValue, NC_BYTE, 1, &fill_value);
-         if (status != NC_ELATEFILL)
-             printf("Error at line %d: expecting NC_ELATEFILL but got %s\n",__LINE__,nc_strerror(status));
+         if (nc_put_att_schar(ncid, varid, _FillValue, NC_BYTE, 1,
+                              &fill_value) != expected_ret) ERR;
          if (nc_close(ncid)) ERR;
 
          /* Open the file and check. */
@@ -533,7 +540,7 @@ main(int argc, char **argv)
       /* THis also works, pointlessly. */
       if (nc_inq_varid(ncid, "temp", NULL)) ERR;
 
-      /* These won't work due to bad paramters. */
+      /* These won't work due to bad parameters. */
       if (nc_rename_var(ncid + MILLION, wind_id, "wind") != NC_EBADID) ERR;
       if (nc_rename_var(ncid, wind_id + TEST_VAL_42, "wind") != NC_ENOTVAR) ERR;
       if (nc_rename_var(ncid, -TEST_VAL_42, "wind") != NC_ENOTVAR) ERR;
@@ -865,7 +872,7 @@ main(int argc, char **argv)
          if (data[i] != data_in[i])
 	    ERR_RET;
 
-      /* These will not work due to bad paramters. */
+      /* These will not work due to bad parameters. */
       if (nc_inq_var_chunking_ints(ncid + MILLION, 0, &storage_in,
                                    chunksize_int_in) != NC_EBADID) ERR;
       if (nc_inq_var_chunking_ints(ncid + TEST_VAL_42, 0, &storage_in,
@@ -1396,7 +1403,7 @@ main(int argc, char **argv)
       if (nc_def_dim(ncid, DIM9_NAME, DIM9_LEN, &dimids[0])) ERR;
       if (nc_def_var(ncid, VAR_NAME9, NC_INT, NDIMS1, dimids, &varid2)) ERR;
 
-      /* These will fail due to bad paramters. */
+      /* These will fail due to bad parameters. */
       if (nc_def_var_deflate(ncid, varid2, 0, 1,
                              NC_MIN_DEFLATE_LEVEL - 1) != NC_EINVAL) ERR;
       if (nc_def_var_deflate(ncid, varid2, 0, 1,
