@@ -1,5 +1,5 @@
 /* This is part of the netCDF package.
-   Copyright 2006 University Corporation for Atmospheric Research/Unidata.
+   Copyright 2018 University Corporation for Atmospheric Research/Unidata.
    See COPYRIGHT file for conditions of use.
 
    This is a very simple example which writes a netCDF file with
@@ -9,7 +9,7 @@
    $Id: tst_utf8.c,v 1.10 2008/10/20 01:48:07 ed Exp $
 */
 
-#include <config.h>
+#include "config.h"
 #include <stdlib.h>
 #include <nc_tests.h>
 #include <err_macros.h>
