@@ -1,5 +1,5 @@
 /*********************************************************************
- *   Copyright 2009, UCAR/Unidata
+ *   Copyright 2018, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
  *********************************************************************/
 
@@ -12,7 +12,7 @@
 #  include <varargs.h>
 #endif
 
-/* nmemonic*/
+/* nmemonics*/
 #define TOPLEVEL 1
 
 /* Forward types */
@@ -56,18 +56,19 @@ typedef struct NCConstant {
 } NCConstant;
 
 typedef struct Datalist {
-    struct Datalist* next; /* chain of all known datalists*/
     int           readonly; /* data field is shared with another Datalist*/
     size_t  length; /* |data| */
     size_t  alloc;  /* track total allocated space for data field*/
-    NCConstant*     data; /* actual list of constants constituting the datalist*/
+    NCConstant**     data; /* actual list of constants constituting the datalist*/
     /* Track various values associated with the datalist*/
     /* (used to be in Constvalue.compoundv)*/
+#if 0
     struct Vlen {
         struct Symbol* schema; /* type/var that defines structure of this*/
         unsigned int count; /* # of vlen basetype instances*/
 	unsigned int uid;       /* unique id for NC_VLEN*/
     } vlen;
+#endif
 } Datalist;
 
 /* Define a structure to track
@@ -75,56 +76,46 @@ typedef struct Datalist {
    In effect, we are parsing the data sequence.
    Push and pop of data sources is supported (see srcpush() below).*/
 typedef struct Datasrc {
-    NCConstant*    data;     /* duplicate pointer; so do not free.*/
+    NCConstant**   data;     /* duplicate pointer; so do not free.*/
     int index;        
     int length;
     int spliced;           /* Was this list spliced into our parent ? */
     struct Datasrc* prev; /* linked list for debugging */
 } Datasrc;
 
+#if 0
 /* Define a holder for passing a start/count array */
 struct Vlendata {
     char* data;
     unsigned long count;
 };
 extern struct Vlendata* vlendata;
-extern Datalist* alldatalists;
+#endif
+
+extern List* alldatalists;
 
 /* from: data.c */
 extern Datalist* builddatalist(int initialize);
-extern void dlfree(Datalist **dlist);
+extern void capture(Datalist* dl);
 extern void dlappend(Datalist*, NCConstant*);
-extern NCConstant builddatasublist(Datalist* dl);
+extern void dlset(Datalist*, size_t, NCConstant*);
+extern NCConstant* builddatasublist(Datalist* dl);
 extern void dlextend(Datalist* dl);
 extern void dlsetalloc(Datalist* dl, size_t newalloc);
+extern Datalist* clonedatalist(Datalist* dl);
+extern void reclaimalldatalists(void);
+extern void reclaimdatalist(Datalist*);
+extern void reclaimconstant(NCConstant*);
 
 int       datalistline(Datalist*);
-#define   datalistith(dl,i) ((dl)==NULL?NULL:((i) >= (dl)->length?NULL:&(dl)->data[i]))
+#define   datalistith(dl,i) ((dl)==NULL?NULL:((i) >= (dl)->length?NULL:(dl)->data[i]))
 #define   datalistlen(dl) ((dl)==NULL?0:(dl)->length)
 
-Datasrc* datalist2src(Datalist* list);
-Datasrc* const2src(NCConstant*);
-NCConstant list2const(Datalist*);
+NCConstant* list2const(Datalist*);
 Datalist* const2list(NCConstant* con);
-void freedatasrc(Datasrc* src);
 
-int issublist(Datasrc* src);
-int isstring(Datasrc* src);
-int isfillvalue(Datasrc* src);
-int istype(Datasrc* src, nc_type);
 int isstringable(nc_type nctype);
 
-void srcpush(Datasrc*);
-void srcpushlist(Datasrc* src, Datalist* cmpd);
-void srcpop(Datasrc*);
-void srcsetfill(Datasrc* ds, Datalist* list);
-
-NCConstant* srcnext(Datasrc*);
-int srcmore(Datasrc*);
-int srcline(Datasrc* ds);
-void srcreset(Datasrc* ds);
-#define srclen(s) ((s)==NULL?0:(s)->length)
-
 #define islistconst(con) ((con)!=NULL && (con)->nctype == NC_COMPOUND)
 #define isfillconst(con) ((con)!=NULL && (con)->nctype == NC_FILLVALUE)
 #define constline(con) (con==NULL?0:(con)->lineno)
@@ -133,9 +124,13 @@ void srcreset(Datasrc* ds);
 #define isnilconst(con) ((con)!=NULL && (con)->nctype == NC_NIL)
 #define   compoundfor(con) ((con)==NULL?NULL:(con)->value.compoundv)
 
-NCConstant* emptystringconst(int,NCConstant*);
+NCConstant* emptycompoundconst(int lineno);
+NCConstant* emptystringconst(int);
 
-NCConstant cloneconstant(NCConstant* con); /* shallow clone*/
+NCConstant* cloneconstant(NCConstant* con); /* deep clone*/
+void clearconstant(NCConstant* con); /* deep clear*/
+#define freeconst(con) freeconstant(con,DEEP);
+void freeconstant(NCConstant* con, int shallow);
 
 void alignbuffer(struct NCConstant* prim, Bytebuffer* buf);
 
@@ -158,23 +153,18 @@ char* word(char* p, Bytebuffer* buf);
 extern Bytebuffer* codebuffer; /* buffer over the std output */
 extern Bytebuffer* stmt; /* single stmt text generation */
 
-#ifdef FASTDATASRC
-#define srcpeek(ds) ((ds)==NULL || (ds)->index >= (ds)->max?NULL:(ds)->data+(ds)->index)
-#else
-NCConstant* srcpeek(Datasrc*);
-#endif
-
 /* Aliases */
 #define srcincr(src) srcnext(src)
 #define srcget(src) srcpeek(src)
 
+extern NCConstant* nullconst(void);
 extern NCConstant nullconstant;
 extern NCConstant fillconstant;
 extern NCConstant nilconstant;
 
 /* From genchar.c */
 void gen_charattr(Datalist*, Bytebuffer*);
-void gen_charvlen(Datalist*, Bytebuffer*);
+void gen_charseq(Datalist*, Bytebuffer*);
 void gen_chararray(struct Dimset*, int, Datalist*, Bytebuffer*, Datalist* fillsrc);
 
 typedef enum ListClass {
@@ -202,5 +192,30 @@ extern void generate_vardata(struct Symbol*, Generator*, Writer writer,Bytebuffe
 extern void generate_basetype(struct Symbol*,NCConstant*,Bytebuffer*,Datalist*,Generator*);
 
 
+/* Obsolete */
+#if 0
+Datasrc* datalist2src(Datalist* list);
+Datasrc* const2src(NCConstant*);
+void freedatasrc(Datasrc* src);
+int issublist(Datasrc* src);
+int isstring(Datasrc* src);
+int isfillvalue(Datasrc* src);
+int istype(Datasrc* src, nc_type);
+void srcpush(Datasrc*);
+void srcpushlist(Datasrc* src, Datalist* cmpd);
+void srcpop(Datasrc*);
+void srcsetfill(Datasrc* ds, Datalist* list);
+NCConstant* srcnext(Datasrc*);
+int srcmore(Datasrc*);
+int srcline(Datasrc* ds);
+void srcreset(Datasrc* ds);
+#define srclen(s) ((s)==NULL?0:(s)->length)
+#ifdef FASTDATASRC
+#define srcpeek(ds) ((ds)==NULL || (ds)->index >= (ds)->max?NULL:(ds)->data+(ds)->index)
+#else
+NCConstant* srcpeek(Datasrc*);
+#endif
+#endif /*0*/
+
 #endif /*DATA_H*/
 
