@@ -1,5 +1,5 @@
 /*********************************************************************
- *   Copyright 1993, UCAR/Unidata
+ *   Copyright 2018, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
  *********************************************************************/
 /* $Id: main.c,v 1.33 2010/05/26 21:43:36 dmh Exp $ */
@@ -14,7 +14,6 @@
 
 #ifdef _MSC_VER
 #include "XGetopt.h"
-#define snprintf _snprintf
 int opterr;
 int optind;
 #endif
@@ -22,13 +21,9 @@ int optind;
 /* Default is netcdf-3 mode 1 */
 #define DFALTCMODE 0
 
-extern void init_netcdf(void);
-extern void parse_init(void);
-extern int ncgparse(void);
-
 /* For error messages */
-char* progname;
-char* cdlname;
+char* progname; /* Global: not reclaimed */
+char* cdlname; /* Global: not reclaimed */
 
 /* option flags */
 int nofill_flag;
@@ -56,14 +51,15 @@ size_t nciterbuffersize;
 
 struct Vlendata* vlendata;
 
-char *netcdf_name; /* command line -o file name */
-char *datasetname; /* name from the netcdf <name> {} || from -N */
+char *netcdf_name = NULL; /* command line -o file name */
+char *datasetname = NULL; /* name from the netcdf <name> {} || from -N */
 
 extern FILE *ncgin;
 
 /* Forward */
 static char* ubasename(char*);
 void usage( void );
+
 int main( int argc, char** argv );
 
 /* Define tables vs modes for legal -k values*/
@@ -202,6 +198,7 @@ usage(void)
 " [-x]"
 " [-N datasetname]"
 " [-L loglevel]"
+" [-H]"
 " [file ... ]",
 	   progname);
     derror("netcdf library version %s", nc_inq_libvers());
@@ -212,10 +209,10 @@ main(
 	int argc,
 	char *argv[])
 {
+    int code = 0;
     int c;
     FILE *fp;
 	struct Languages* langs;
-    char* lang_name;
 #ifdef __hpux
     setlocale(LC_CTYPE,"");
 #endif
@@ -223,8 +220,8 @@ main(
     init_netcdf();
 
     opterr = 1;			/* print error message if bad option */
-    progname = ubasename(argv[0]);
-    cdlname = "-";
+    progname = nulldup(ubasename(argv[0]));
+    cdlname = NULL;
     netcdf_name = NULL;
     datasetname = NULL;
     l_flag = 0;
@@ -247,11 +244,6 @@ main(
 #endif
     memset(&globalspecials,0,sizeof(GlobalSpecialData));
 
-#if _CRAYMPP && 0
-    /* initialize CRAY MPP parallel-I/O library */
-    (void) par_io_init(32, 32);
-#endif
-
     while ((c = getopt(argc, argv, "134567bB:cdD:fhHk:l:M:no:Pv:xL:N:")) != EOF)
       switch(c) {
 	case 'd':
@@ -283,14 +275,15 @@ main(
 	  }
 	  l_flag = L_BINARY;
 	  break;
-	case 'h':
+	case 'H':
 	  header_only = 1;
 	  break;
-	case 'H':
+	case 'h':
 	  usage();
-	  exit(0);
+	  goto done;
         case 'l': /* specify language, instead of using -c or -f or -b */
 	{
+            char* lang_name = NULL;
 	    if(l_flag != 0) {
               fprintf(stderr,"Please specify only one language\n");
               return 1;
@@ -299,18 +292,24 @@ main(
               derror("%s: output language is null", progname);
               return(1);
             }
+#if 0
+            lang_name = estrdup(optarg);
+#endif
             lang_name = (char*) emalloc(strlen(optarg)+1);
-	    (void)strcpy(lang_name, optarg);
-	    for(langs=legallanguages;langs->name != NULL;langs++) {
+            (void)strcpy(lang_name, optarg);
+
+            for(langs=legallanguages;langs->name != NULL;langs++) {
               if(strcmp(lang_name,langs->name)==0) {
-	  	l_flag = langs->flag;
+                l_flag = langs->flag;
                 break;
               }
-	    }
+            }
 	    if(langs->name == NULL) {
               derror("%s: output language %s not implemented",progname, lang_name);
+              nullfree(lang_name);
               return(1);
 	    }
+            nullfree(lang_name);
 	}; break;
 	case 'L':
 	    ncloglevel = atoi(optarg);
@@ -324,9 +323,11 @@ main(
           binary_ext = ".cdf";
 	  break;
 	case 'o':		/* to explicitly specify output name */
+	  if(netcdf_name) efree(netcdf_name);
 	  netcdf_name = nulldup(optarg);
 	  break;
 	case 'N':		/* to explicitly specify dataset name */
+	  if(datasetname) efree(datasetname);
 	  datasetname = nulldup(optarg);
 	  break;
 	case 'x': /* set nofill mode to speed up creation of large files */
@@ -351,23 +352,18 @@ main(
                        5 (=> classic 64 bit data aka CDF-5)
 		   */
 	    struct Kvalues* kvalue;
-	    char *kind_name = (optarg != NULL
-				? (char *) emalloc(strlen(optarg)+1)
-				: emalloc(1));
-	    if (! kind_name) {
-		derror ("%s: out of memory", progname);
-		return(1);
-	    }
-            if(optarg != NULL)
-              (void)strcpy(kind_name, optarg);
+            if(optarg == NULL) {
+                derror("-k flag has no value");
+		return 2;
+            }
             for(kvalue=legalkinds;kvalue->name;kvalue++) {
-              if(strcmp(kind_name,kvalue->name) == 0) {
-                k_flag = kvalue->k_flag;
-                break;
-              }
+                if(strcmp(optarg,kvalue->name) == 0) {
+                  k_flag = kvalue->k_flag;
+                  break;
+                }
             }
             if(kvalue->name == NULL) {
-                derror("Invalid format: %s",kind_name);
+                derror("Invalid format: %s",optarg);
                 return 2;
             }
 	} break;
@@ -417,21 +413,21 @@ main(
     }
 
 #ifndef ENABLE_C
-    if(c_flag) {
+    if(l_flag == L_C) {
 	  fprintf(stderr,"C not currently supported\n");
-	  exit(1);
+	  code=1; goto done;
     }
 #endif
 #ifndef ENABLE_BINARY
     if(l_flag == L_BINARY) {
 	  fprintf(stderr,"Binary netcdf not currently supported\n");
-	  exit(1);
+	  code=1; goto done;
     }
 #endif
 #ifndef ENABLE_JAVA
     if(l_flag == L_JAVA) {
 	  fprintf(stderr,"Java not currently supported\n");
-	  exit(1);
+	  code=1; goto done;
     }
 #else
     if(l_flag == L_JAVA && mainname != NULL && strcmp(mainname,"main")==0)
@@ -440,7 +436,7 @@ main(
 #ifndef ENABLE_F77
     if(l_flag == L_F77) {
 	  fprintf(stderr,"F77 not currently supported\n");
-	  exit(1);
+	  code=1; goto done;
     }
 #endif
 
@@ -486,13 +482,12 @@ main(
 		break;
 	    }
 	}
+    }
 
-	cdlname = (char*)emalloc(NC_MAX_NAME);
-	cdlname = nulldup(argv[0]);
-	if(cdlname != NULL) {
-	  if(strlen(cdlname) > NC_MAX_NAME)
-	    cdlname[NC_MAX_NAME] = '\0';
-	}
+    cdlname = nulldup(argv[0]);
+    if(cdlname != NULL) {
+	if(strlen(cdlname) > NC_MAX_NAME)
+	  cdlname[NC_MAX_NAME] = '\0';
     }
 
     parse_init();
@@ -503,7 +498,7 @@ main(
 
     /* Compute the k_flag (1st pass) using rules in the man page (ncgen.1).*/
 
-#ifndef USE_CDF5
+#ifndef ENABLE_CDF5
     if(k_flag == NC_FORMAT_CDF5) {
       derror("Output format CDF5 requested, but netcdf was built without cdf5 support.");
       return 0;
@@ -586,7 +581,11 @@ main(
     if(!syntax_only && error_count == 0)
         define_netcdf();
 
-    return 0;
+done:
+    nullfree(netcdf_name);
+    nullfree(datasetname);
+    finalize_netcdf(code);
+    return code;
 }
 
 void
@@ -600,3 +599,10 @@ init_netcdf(void) /* initialize global counts, flags */
     stmt = bbNew();
     error_count = 0; /* Track # of errors */
 }
+
+void
+finalize_netcdf(int retcode)
+{
+    nc_finalize();
+    exit(retcode);
+}
