@@ -1,4 +1,4 @@
-/* Copyright 2009, UCAR/Unidata and OPeNDAP, Inc.
+/* Copyright 2018, UCAR/Unidata and OPeNDAP, Inc.
    See the COPYRIGHT file for more information. */
 
 #include "config.h"
@@ -95,7 +95,7 @@ oc_close(OCobject link)
 This procedure is used to send requests to the server
 to obtain either a DAS, DDS, or DATADDS response
 and produce a corresponding tree.
-It fetchs and parses a given class of DXD the server specified
+It fetches and parses a given class of DXD the server specified
 at open time, and using a specified set of constraints
 and flags.
 
@@ -1452,7 +1452,7 @@ and the read request cannot be completed.
 
 OCerror
 oc_data_readn(OCobject link, OCobject datanode,
-                 size_t* start, size_t N,
+                 const size_t* start, size_t N,
 	         size_t memsize, void* memory)
 {
     OCerror ocerr = OC_NOERR;
