@@ -1,3 +1,4 @@
+#include "config.h"
 #include <sys/types.h>
 #include <stdlib.h>
 #include <string.h>
@@ -7,6 +8,8 @@
 /* Older versions of the hdf library may define H5PL_type_t here */
 #include <H5PLextern.h>
 
+
+
 #ifndef DLL_EXPORT
 #define DLL_EXPORT
 #endif
@@ -90,7 +93,11 @@ size_t H5Z_filter_bzip2(unsigned int flags, size_t cd_nelmts,
 
     /* Prepare the output buffer. */
     outbuflen = nbytes * 3 + 1;  /* average bzip2 compression ratio is 3:1 */
+#ifdef HAVE_H5ALLOCATE_MEMORY
     outbuf = H5allocate_memory(outbuflen,0);
+#else
+    outbuf = (char*)malloc(outbuflen * sizeof(char));
+#endif
     if (outbuf == NULL) {
       fprintf(stderr, "memory allocation failed for bzip2 decompression\n");
       goto cleanupAndFail;
@@ -123,7 +130,11 @@ size_t H5Z_filter_bzip2(unsigned int flags, size_t cd_nelmts,
       if (ret != BZ_STREAM_END && stream.avail_out == 0) {
         /* Grow the output buffer. */
         newbuflen = outbuflen * 2;
+#ifdef HAVE_H5RESIZE_MEMORY
         newbuf = H5resize_memory(outbuf, newbuflen);
+#else
+        newbuf = realloc(outbuf,newbuflen);
+#endif
         if (newbuf == NULL) {
           fprintf(stderr, "memory allocation failed for bzip2 decompression\n");
           goto cleanupAndFail;
@@ -167,7 +178,12 @@ size_t H5Z_filter_bzip2(unsigned int flags, size_t cd_nelmts,
 
     /* Prepare the output buffer. */
     outbuflen = nbytes + nbytes / 100 + 600;  /* worst case (bzip2 docs) */
+#ifdef HAVE_H5ALLOCATE_MEMORY
     outbuf = H5allocate_memory(outbuflen,0);
+#else
+    outbuf = (char*)malloc(outbuflen * sizeof(char));
+#endif
+
     if (outbuf == NULL) {
       fprintf(stderr, "memory allocation failed for bzip2 compression\n");
       goto cleanupAndFail;
@@ -185,13 +201,23 @@ size_t H5Z_filter_bzip2(unsigned int flags, size_t cd_nelmts,
   }
 
   /* Always replace the input buffer with the output buffer. */
+#ifdef HAVE_H5FREE_MEMORY
   H5free_memory(*buf);
+#else
+  free(*buf);
+#endif
+
   *buf = outbuf;
   *buf_size = outbuflen;
   return outdatalen;
 
  cleanupAndFail:
   if (outbuf)
+#ifdef HAVE_H5FREE_MEMORY
     H5free_memory(outbuf);
+#else
+  free(outbuf);
+#endif
+
   return 0;
 }
